import base64
import json
from appPublic.rsawrap import RSA
from os import urandom
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.primitives.asymmetric import padding as asym_padding
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend

def write_pubkey_pem(pubkey, pemfile):
	with open(pemfile, "wb") as f:
		f.write(
			pubkey.public_bytes(
				encoding=serialization.Encoding.PEM,
				format=serialization.PublicFormat.SubjectPublicKeyInfo
			)
		)

def write_prikey_pem(prikey, pemfile):
	with open(pemfile, "wb") as f:
		f.write(
			prikey.private_bytes(
				encoding=serialization.Encoding.PEM,
				#format=serialization.PrivateFormat.TraditionalOpenSSL, 
				format=serialization.PrivateFormat.PKCS8,  # 指定为 PKCS#8 格式
				encryption_algorithm=serialization.NoEncryption()       # 可设置加密
			)
		)

def get_rsa_pubkey_from_pemstr(pemstr):
	# 加载公钥
	pem = pemstr.encode('utf-8')
	public_key = serialization.load_pem_public_key(pem)
	return public_key

def get_rsa_prikey_from_pemstr(pemstr):
	pem = pemstr.encode('utf-8')
	private_key = serialization.load_pem_private_key(pem, password=None)
	return private_key

def get_rsa_pubkey_from_b64keystring(b64ks):
	der_data = base64.b64decode(b64ks)
	# 加载公钥
	public_key = serialization.load_der_public_key(der_data, backend=default_backend())
	return public_key

def pubkey_encode(pubkey, bin):
	ciphertext = pubkey.encrypt(
		bin,
		asym_padding.PKCS1v15()
	)

	# 输出 base64 编码后的密文
	ciphertext_b64 = base64.b64encode(ciphertext).decode()
	return ciphertext_b64

def get_rsa_prikey_from_b64keystring(b64ks):
	der_data = base64.b64decode(b64ks)
	private_key = serialization.load_der_private_key(der_data, password=None, backend=default_backend())
	return private_key

def prikey_decode(prikey, text):
	ciphertext = base64.b64decode(text)
	plaintext = prikey.decrypt(
		ciphertext,
		asym_padding.PKCS1v15()
	)
	return plaintext

def generate_key(key_size=32):
    return urandom(key_size)  # 32 字节 = 256 位

def pad(data: bytes) -> bytes:
    padder = padding.PKCS7(128).padder()  # AES 块大小是 128 bits
    return padder.update(data) + padder.finalize()

def unpad(data: bytes) -> bytes:
    unpadder = padding.PKCS7(128).unpadder()
    return unpadder.update(data) + unpadder.finalize()

def aes_encrypt_ecb(key: bytes, plaintext: str) -> bytes:
    cipher = Cipher(algorithms.AES(key), modes.ECB(), backend=default_backend())
    encryptor = cipher.encryptor()
    padded_data = pad(plaintext.encode())
    return encryptor.update(padded_data) + encryptor.finalize()

def aes_decrypt_ecb(key: bytes, ciphertext: bytes) -> str:
    cipher = Cipher(algorithms.AES(key), modes.ECB(), backend=default_backend())
    decryptor = cipher.decryptor()
    padded_plain = decryptor.update(ciphertext) + decryptor.finalize()
    return unpad(padded_plain).decode()

def data_jj_encode(data, pubkey_b64str):
	""" return a dict with:
		AESKey:aes key encrypted by public key
		data: data encrypted by aes key
		步骤说明：
		1. 从pemstr生成公钥
		2. 生成随机的aes密钥
		3. 用公钥加密aes密钥到aes_key_cyber
		4. 将data数据用json转换为字符串js
		5. 用aes密钥将js字符串加密到data_cyber
		6. 将data_cyber用base64编码到data_cyber_b64
		7. 构造返回字典：{
			'AESKey': aes_key_cyber,
			'data': data_cyber_b64
		}，并返回
	"""
	pubkey = get_rsa_pubkey_from_pemstr(pubkey_b64str)
	aes_key = generate_key()
	print(f'{aes_key=}')
	aes_key_cyber  = pubkey_encode(pubkey, aes_key)
	
	jd = json.dumps(data)
	data_cyber = aes_encrypt_ecb(aes_key, jd)
	data_cyber_b64 = base64.b64encode(data_cyber).decode('utf-8')
	return {
		'AESKey': aes_key_cyber,
		'data': data_cyber_b64
	}
	
def data_jj_decode(cdata, prikey_b64str):
	prikey = get_rsa_prikey_from_pemstr(prikey_str)
	aes_key = prikey_decode(prikey, cdata['AESKey'])
	print(f'{aes_key=}')
	js_cyber = base64.b64decode(cdata['data'])
	js = aes_decrypt_ecb(aes_key, js_cyber)
	d = json.loads(js)
	return d

if __name__ == '__main__':
	# 示例
	import sys

	key = generate_key()
	if len(sys.argv) > 1:
		key = base64.b64decode(sys.argv[1])
	key_b64 = base64.b64encode(key)

	plaintext = "互信是合作与发展的基础"
	ciphertext = aes_encrypt_ecb(key, plaintext)
	cb64 = base64.b64encode(ciphertext)
	print("原文：", plaintext, "密文（base64ed）：", cb64, "KEY:", base64.b64encode(key))
	decrypted = aes_decrypt_ecb(key, ciphertext)
	print("解密结果：", decrypted)
	d =  {
		"field1":"19ygf23",
		"filed2": 2,
		"field3": "r32g"
	}
	pubkey_str="-----BEGIN PUBLIC KEY-----\nMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA3LWvOQCufDQpf9PsOrBF" + \
       "SsjPqIg/x1YZpQ4MksZs4BwHWjHT9PHoJ88OZl2cXmyaY6Hde2KXG1v873lI70/2" + \
       "oUizX8Fyl0TTliAQ/p/kSsmgHfzhY+KOZUQ5HwXwnjFKv1q+Y0PpseNPFFHGiB1H" + \
       "/6p8kJeV61GN00etwvAvDdfZKm9W6t1ByD9Bsr3/VSHJtv1DqVdgmVMxnI7W4mx6" + \
       "UmseGmywngLvBHXI0+MWRK1RszCMISA6SsKTlT9IdT/r64vLht+NImEY6KPwODpu" + \
       "6vBUSyRqbGVitZ6PPIPwzLkB8LZ69eTjK5SMXP6RD7kHio+vb4ljHWXzDfRH9HXq" + \
       "FQIDAQAB\n-----END PUBLIC KEY-----"
	
	prikey_str = (
		"-----BEGIN PRIVATE KEY-----\n"
		"MIIEvwIBADANBgkqhkiG9w0BAQEFAASCBKkwggSlAgEAAoIBAQDcta85AK58NCl/"
		"0+w6sEVKyM+oiD/HVhmlDgySxmzgHAdaMdP08egnzw5mXZxebJpjod17YpcbW/zv"
		"eUjvT/ahSLNfwXKXRNOWIBD+n+RKyaAd/OFj4o5lRDkfBfCeMUq/Wr5jQ+mx408U"
		"UcaIHUf/qnyQl5XrUY3TR63C8C8N19kqb1bq3UHIP0Gyvf9VIcm2/UOpV2CZUzGc"
		"jtbibHpSax4abLCeAu8EdcjT4xZErVGzMIwhIDpKwpOVP0h1P+vri8uG340iYRjo"
		"o/A4Om7q8FRLJGpsZWK1no88g/DMuQHwtnr15OMrlIxc/pEPuQeKj69viWMdZfMN"
		"9Ef0deoVAgMBAAECggEAAWQPt58+ej/l5w1R1sjXlvIPBPPe2wW64aRmBeLEOuFS"
		"Voor2MZRfTa7039RhfLPQkN53tYvN5frzyKHiCKL3lZT8w2ICFYBs1d+Y+xkBuig"
		"XrACR5vc49x7M6vY6VZML0qGGqAJ7/oLvIgLZT4x/vV3FX4vDq2T5Dn2aV0UHtUz"
		"ZSwJHJLTqIyHSSzAJetW8cEoEE1u3g7Wu3K2+7Z7nRqQzXnpC3+4wNGV+C7/NYWr"
		"8VEf8ygRT0Znom0DIWHnAS4PCJFYefkQfxaDqAA3JSUj3BrbnFJzC4+sSPSmPueL"
		"6HWubeZyIiUud7h5mGZS/VNylCp/92gvumJ8YeKA4QKBgQDvbXa0TFXgLNaunf6T"
		"PK7dFSZE2gIGyDhRNsAWvDXQITmfQDe5cwI1++tMnsULDPhgEmfORHep00L8JIv4"
		"uUArKQ9miI3rfrD2NAY5SbqUqtZA9Q5FtfyouTkKWaEvOt2jAYLTX1m+EYtk4GYE"
		"X36VrigU7NhuVQlQM+6M2c5BbQKBgQDr/Iyaza0MdD3oL2EBELVVVEukBs/6Dghp"
		"NS6PiRny+KznlPYJgowIqN+JXgxxyq+yaPSWBFF+5gJv39+YofHgUkDBlVQVWtRq"
		"g6QSWdBX24I1RsPhCW6yioSbbmTNAPCP5iiQ6a50slMoKs+FU+t7YCmtWH4LxmhU"
		"ZVtqSxMKSQKBgQDFFAxmooL5cM8RkrlvBr9E+dizSePrIznAF9oLnMaK2B5vKFP2"
		"EWXbe86WFgZJb1au9dd+YLtDDcOQeqDAjfP00up7UqHHG1Scg/wvgbHUF78eXtht"
		"pzwewb/i9h4IG2M+fsrqJA1ht11PHqq6pMQ0ftM9TitkvhhZbTXJuKrfDQKBgQCQ"
		"W0s/nzWOesUr9V+noSrr6cbjre8njdIDWa8DyuB+yzQp7eub4an72iH0C/4c1doo"
		"jx+fbUDVGaq/+Pj0i4xEpsxKHyjybz+lcKW7PAnA1NyP7pwAWx7GRQndcuhip4KA"
		"5FBIZUp8xGEuHxIXA0D3ldmo2oswdMx5ZA9JtGcPUQKBgQDtlyIeU8p9wTgCLYn8"
		"PVrVMJNiLv4zvJdmQNkBQmNIC67mLUGcZRZRfskk4hYB8sglAGkHZIUBDMFk785+"
		"grB5Y0Rn7RAuzVs6uoibwAx4u3rEjENR0gOdyzMATzqjgWL8GnrG5QU6VHRIVwr6"
		"ni/+cSjJYn6PolxPdurAZXYhqQ==\n"
		"-----END PRIVATE KEY-----"
	)
	pubkey = get_rsa_pubkey_from_pemstr(pubkey_str)
	prikey = get_rsa_prikey_from_pemstr(prikey_str)
	key_crypt = pubkey_encode(pubkey, key_b64)
	key_b64T = prikey_decode(prikey, key_crypt)
	print(f"RSA加密{key_b64=}==>{key_crypt=}, \n解密==>{key_b64T=}")
	x = data_jj_encode(d, pubkey_str)
	d1 = data_jj_decode(x, prikey_str)
	print(f'{d=}, {d1=}')
	print(json.dumps(x, indent=4))

